#!/bin/ksh
#####################################################################################################
#
# Shell script to monitor the file systems on the AIX server. It will read from a properties file
# named space_monitor_config.properties resides in the same directory of this shell script.
# The properties file contains the configuration information regarding what file systems
# you want to be monitored, each line defines the file system and it's threshold of used
# percentage in the format of:  filesystem=percentage%(example: /apps/awl0/log=90%  this
# would mean once the file system/mounting point space is reaching 90% full and over, an Alert
# email would be sent to the email list defined in the same properties file:
# ALERT_EMAIL_LIST=fang@abc.com;mike.burk@abc.com
#
# If the properties file is not existing or not readable, it will prompt the user if it should run
# in default mode, in which all the file systems will be monitored and if there's any reaching
# 90% full or over, email alert will be sent to an address in the code below.(DEFAULT_EMAIL_LIST)
# The default mode function is more for Unix admin to use for general purpose of space monitoring.
# Currently the default mode function is disabled to provide simplicity for the end user.
#
# This is the shell does the major job. 
#
# Config file     : space_monitor_config.properties
#
# Operating system: AIX Version 5.3
# Initial Version : Fang
# Date            : Oct 20, 2009
#
# Change History:
#   Date               Name                Summary
# ---------    --------------------     ----------------------------------------------
#
#
# 
#####################################################################################################

PROPERTIES_FILE="./space_monitor_config.properties"

#if no properties file define the file system list and emails then
#default set 90% full alert and send email to DEFAULT_EMAIL_LIST	
#and the default checking interval would be 15 minutes(900 seconds)
DEFAULT_ALERT=90
DEFAULT_EMAIL_LIST="fang@abc.com"
DEFAULT_MONITOR_INTERVAL=900 

function isNumber {
  if [[ "$1" == +([0-9]) ]]
  then
       echo "true"
  else 
       echo "false"
  fi

}

function check_space_usage {
while read input
do

  percentage=`echo $input | awk '{ print $1}' | cut -d'%' -f1`
  partition=$(echo $input | awk '{ print $2}')


  if [ `isNumber $percentage` == "true" ]  #the percentage column could be non-numeric in the output of "df -g"
  then                                     #so test it to make sure.
        if [ $# -eq 0 ]
        then
              #Function is called without any argument

              #if DEFAULT_ALERT is defined in the main program then we use it
              #otherwise we assume it is 90%.Same as EMAIL_LIST below.
              #this is just to make this function not having dependency on the main program
              if [ -z $DEFAULT_ALERT ]
              then  
                    echo "Default alert level is not defined. Assume 90% as threshold.\n"
                    threshold=90 
              else
                    threshold=$DEFAULT_ALERT
              fi


              if [ $percentage -ge $threshold ]
              then
                    usage=$(df -g $partition)
                    echo "File system exceeds $threshold%"
 
                    if [ -z "$DEFAULT_EMAIL_LIST" ]
                    then 
                          EMAIL_LIST="phil.fang@rbc.com"
                    else
                          EMAIL_LIST=$DEFAULT_EMAIL_LIST
                    fi

                    echo "Below is the file system usage:\n\n$usage" |  \
                    mail -s "Alert: $partition is reaching $percentage% full on server $(hostname), $(date) " $EMAIL_LIST
               fi

         else

               #function is called with file system names and email list passed in

               #extract the definition of the file system
               thePartitionSpecified=$(echo $1 | grep $partition"=" )
               echo "\n\nThe definition in properties file:$thePartitionSpecified."

               #get the percentage specified in the properties file for the partition
               threshold=$(echo "$thePartitionSpecified" | cut -d '=' -f 2 | cut -d '%' -f 1)
               echo "threshold defined: $threshold%"

               if [ $percentage -ge $threshold ]
               then

                     echo "File system $partition has used $percentage%, exceeds threshold defined at $threshold% !!!\n"
                     usage=$(df -g $partition)

                     echo "Below is the file system usage:\n\n$usage" |  \
                     mail -s "Alert: $partition is reaching $percentage% full on server $(hostname), $(date) " $2
               
               else

                     echo "File system $partition has used $percentage%, not exceeding threshold defined at $threshold%.\n"
               fi

         fi #end test $1,the function argument exists or not
  fi #end test isNumber

 done

}



##########################################################
#
# Main logic starts from here
#
##########################################################


if [ ! -a $PROPERTIES_FILE  -o ! -r $PROPERTIES_FILE  ] 
then

#     Disable the capability to run in default mode for now cause not sure if it's useful

#     line1="####################################################################"
#     line2="\nConfiguration file not existed or is not readable: $PROPERTIES_FILE !"
#     line3="\nWithout configuration this monitor program will run in default mode in which "
#     line4="\n it monitors all file systems and send alert email when there is file system 90% full.\n"
#     line5="\n\n####################################################################"
#     line6="\n\n Do you want to do that(y/n)? : \c" 

#     echo "$line1$line2$line3$line4$line5$line6"
#     read userinput
#     if [ $userinput == "y" ]
#     then 
#           #excluding the first line of the df output and pass in only the 4th/7th columns
#           #into the function
#           df -g | grep -E -v "^Filesystem" | awk '{print $4 " "  $7}' | check_space_usage      
#           sleep $DEFAULT_MONITOR_INTERVAL
#     else
#           echo "\n\n*****************The file system space monitoring program is not running !***************** \n"
           
#     fi

#     exit 0

   
      echo "*************The configuration properties file doesnot exist or it is not readable.**************"
      echo "*************Please double check it is in the same directory and readable and then try again.**************\n"

      exit 0

fi
 

echo "\n\n**************************************************************************************************"
echo "              Space monitor program started at : `date` " 
echo "              and  running with configuration in $PROPERTIES_FILE."
echo "**************************************************************************************************\n\n"


#extract the file system list defined in the properties file that we 
#want to watch for their space usage, into a variable and separate 
#them by a "|", it will be used as a regular expression later.
counter=1
for i in `cat $PROPERTIES_FILE | grep -v "#"  `
do
   if [[ "$i" == *ALERT_EMAIL_LIST* ]]
   then

        DEFAULT_EMAIL_LIST=`echo $i | cut -f 2 -d "=" | sed 's/;/ /g' `

        echo "Email list defined: $DEFAULT_EMAIL_LIST"


   elif [[ "$i" == *MONITOR_INTERVAL* ]]
   then

        DEFAULT_MONITOR_INTERVAL=`echo $i | sed 's/ //g' | cut -f 2 -d "=" `
        
        echo "Checking Interval defined:$DEFAULT_MONITOR_INTERVAL seconds."       
  

   elif [[ "$i" == *% ]]
   then

        FILE_SYSTEM=`echo $i | sed 's/ //g' | cut -f 1 -d "="`

        #do this check so we don't add a "|" at the beginning of the list
        if [ $counter -eq 1 ]
        then
             LIST_TO_CHECK_REG_EXPR=$LIST_TO_CHECK_REG_EXPR$FILE_SYSTEM$
        else
             LIST_TO_CHECK_REG_EXPR=$LIST_TO_CHECK_REG_EXPR"|"$FILE_SYSTEM$
        fi
   
        counter=0

        CONFIG_LIST=$CONFIG_LIST$i"\n"
        
   fi

done

echo "\nFile config list:\n$CONFIG_LIST"


#List out the file system usage information and only let those we defined in the
#properties file be passed in the function for checking.
df -g | grep -E "$LIST_TO_CHECK_REG_EXPR" | awk '{print $4 " "  $7}' | check_space_usage $CONFIG_LIST $DEFAULT_EMAIL_LIST


echo "Sleep $DEFAULT_MONITOR_INTERVAL seconds."
sleep $DEFAULT_MONITOR_INTERVAL
